#ifndef _MOTION_MANAGER_H
#define _MOTION_MANAGER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "stm32f4xx_hal.h"

#define MANAGER_MAX_NUM_SERVOS          31

typedef enum{MM_NONE          = 0,
             MM_ACTION        = 1,
             MM_WALKING       = 2,
             MM_JOINTS        = 3} TModules;

// servo information structure
typedef struct{
  uint8_t id;
  uint16_t model;
  uint16_t encoder_resolution;
  uint8_t gear_ratio;
  uint16_t max_angle;
  uint16_t center_angle;
  uint16_t max_speed;
  int16_t current_angle;
  uint16_t current_value;
  TModules module;
  uint8_t enabled;
  int16_t cw_angle_limit;
  int16_t ccw_angle_limit;
  uint8_t dyn_version;
}TServoInfo;

// public variables
extern int64_t current_angles[MANAGER_MAX_NUM_SERVOS];

// public functions
void manager_init(uint16_t period_us);
inline uint16_t manager_get_period(void);
inline void manager_set_period(uint16_t period_us);
inline void manager_enable(void);
inline void manager_disable(void);
inline uint8_t manager_is_enabled(void);
void manager_enable_balance(void);
void manager_disable_balance(void);
inline uint8_t manager_get_num_servos(void);
inline void manager_set_module(uint8_t servo_id,TModules module);
inline TModules manager_get_module(uint8_t servo_id);
inline void manager_enable_servo(uint8_t servo_id);
inline void manager_disable_servo(uint8_t servo_id);
inline uint8_t manager_is_servo_enabled(uint8_t servo_id);
inline int16_t manager_get_cw_angle_limit(uint8_t servo_id);
inline int16_t manager_get_ccw_angle_limit(uint8_t servo_id);
// operation functions
uint8_t manager_in_range(unsigned short int address, unsigned short int length);
void manager_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data);

#ifdef __cplusplus
}
#endif

#endif
